=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2010-06-30 12:13:14 +0000
+++ bzrlib/builtins.py	2010-07-08 11:47:56 +0000
@@ -2712,6 +2712,17 @@
                 "NAME_PATTERN or --default-rules.")
         name_pattern_list = [globbing.normalize_pattern(p)
                              for p in name_pattern_list]
+        bad_patterns = []
+        for p in name_pattern_list:
+            if not globbing.Globster.is_pattern_valid(p):
+                bad_patterns.append(p)
+        if bad_patterns:
+            msg = 'Invalid ignore patterns found.'
+            pats = ''
+            for p in bad_patterns:
+                pats += ('\n  %s' % p)
+            ui.ui_factory.show_error(msg + pats)
+            raise errors.InvalidPattern('')
         for name_pattern in name_pattern_list:
             if (name_pattern[0] == '/' or
                 (len(name_pattern) > 1 and name_pattern[1] == ':')):

=== modified file 'bzrlib/globbing.py'
--- bzrlib/globbing.py	2010-07-08 05:11:16 +0000
+++ bzrlib/globbing.py	2010-07-08 11:47:56 +0000
@@ -179,24 +179,44 @@
     so are matched first, then the basename patterns, then the fullpath
     patterns.
     """
+    TYPE_FULLPATH = 1
+    TYPE_BASENAME = 2
+    TYPE_EXTENSION = 3
+
+    translators = {
+        TYPE_FULLPATH : _sub_fullpath,
+        TYPE_BASENAME : _sub_basename,
+        TYPE_EXTENSION : _sub_extension,
+    }
+
+    # Prefixes used to combine various patterns.
+    # See: Globster._add_patterns
+    PREFIX_EXTENSION = r'(?:.*/)?(?!.*/)(?:.*\.)'
+    PREFIX_BASENAME = r'(?:.*/)?(?!.*/)'
+    PREFIX_FULLPATH = r''
+
+    prefixes = {
+        TYPE_FULLPATH : PREFIX_FULLPATH,
+        TYPE_BASENAME : PREFIX_BASENAME,
+        TYPE_EXTENSION : PREFIX_EXTENSION,
+    }
+
     def __init__(self, patterns):
         self._regex_patterns = []
         path_patterns = []
         base_patterns = []
         ext_patterns = []
+        pattern_lists = {
+            Globster.TYPE_FULLPATH : path_patterns,
+            Globster.TYPE_EXTENSION : ext_patterns,
+            Globster.TYPE_BASENAME : base_patterns,
+        }
         for pat in patterns:
             pat = normalize_pattern(pat)
-            if pat.startswith(u'RE:') or u'/' in pat:
-                path_patterns.append(pat)
-            elif pat.startswith(u'*.'):
-                ext_patterns.append(pat)
-            else:
-                base_patterns.append(pat)
-        self._add_patterns(ext_patterns,_sub_extension,
-            prefix=r'(?:.*/)?(?!.*/)(?:.*\.)')
-        self._add_patterns(base_patterns,_sub_basename,
-            prefix=r'(?:.*/)?(?!.*/)')
-        self._add_patterns(path_patterns,_sub_fullpath)
+            pattern_lists[Globster.identify(pat)].append(pat)
+        for k, v in pattern_lists.iteritems():
+            self._add_patterns(v, Globster.translators[k],
+                Globster.prefixes[k])
 
     def _add_patterns(self, patterns, translator, prefix=''):
         while patterns:
@@ -225,6 +245,41 @@
             raise e
         return None
 
+    @staticmethod
+    def identify(pattern):
+        """Returns pattern category.
+
+        :param pattern: normalized pattern.
+        Identify if a pattern is fullpath, basename or extension
+        and returns the appropriate type.
+        """
+        if pattern.startswith(u'RE:') or u'/' in pattern:
+            return Globster.TYPE_FULLPATH
+        elif pattern.startswith(u'*.'):
+            return Globster.TYPE_EXTENSION
+        else:
+            return Globster.TYPE_BASENAME
+
+    @staticmethod
+    def is_pattern_valid(pattern):
+        """Returns True if pattern is valid.
+
+        :param pattern: Normalized pattern.
+        is_pattern_valid() assumes pattern to be normalized.
+        see: globbing.normalize_pattern
+        """
+        result = True
+        translator = Globster.translators[Globster.identify(pattern)]
+        tpattern = '(%s)' % translator(pattern)
+        try:
+            re_obj = re.compile(tpattern, re.UNICODE)
+            re_obj.search("") # force compile
+        except errors.InvalidPattern, e:
+            result = False
+        return result
+
+
+
 class ExceptionGlobster(object):
     """A Globster that supports exception patterns.
     
@@ -272,14 +327,9 @@
         self._regex_patterns = []
         for pat in patterns:
             pat = normalize_pattern(pat)
-            if pat.startswith(u'RE:') or u'/' in pat:
-                self._add_patterns([pat], _sub_fullpath)
-            elif pat.startswith(u'*.'):
-                self._add_patterns([pat], _sub_extension,
-                    prefix=r'(?:.*/)?(?!.*/)(?:.*\.)')
-            else:
-                self._add_patterns([pat], _sub_basename,
-                    prefix=r'(?:.*/)?(?!.*/)')
+            pat_type = Globster.identify(pat)
+            self._add_patterns([pat], Globster.translators[pat_type],
+                Globster.prefixes[pat_type])
 
 
 _slashes = re.compile(r'[\\/]+')

